HC = require "Libs.hardoncollider"
require "Libs.TESound"
require "Libs.AnAL"
require "tablemanager"
require "game"
require "player"
require "menu"
require "editor"
function love.load()
	--love.window.setMode(768, 768, {fullscreen=true, minwidth=768, minheight=768})
	rotate = false
	--canvas = love.graphics.newCanvas(768, 768)
	Z = false
	Debug.load()
	Game.load()
	Menu.load()
	Player.load()
	Editor.load()
end
function love.update(dt)
	Debug.update(dt)
	collider:update(dt)
	if rotate == true then
		pointer.y, pointer.x = love.mouse:getPosition()
		pointer.x = 768 - pointer.x
		-- pointer.x = pointer.x * window.vscale
		-- pointer.y = pointer.y * window.hscale
	else
		pointer.x, pointer.y = love.mouse:getPosition()
		-- pointer.x = pointer.x * window.hscale
		-- pointer.y = pointer.y * window.vscale
	end
	if game.state == "title" then
		Menu.update(dt)
		pointer.point:moveTo(pointer.x, pointer.y)
	elseif game.state == "game" then
		--love.graphics.setCanvas(canvas)
		--canvas:clear()
		Game.update(dt)
	elseif game.state == "game over" then
		GameOver.update(dt)
	elseif game.state == "congratulations" then
		Congrats.update(dt)
	elseif game.state == "score" then
		Score.update(dt)
	elseif game.state == "editor" then
		Editor.update(dt)
	elseif game.state == "test" then
		Test.update(dt)
	end
	TEsound.cleanup()
end
function love.draw()
	love.graphics.setColor(game.red, game.green, game.blue, 255)
	for i, v in ipairs(background.grid) do
		love.graphics.draw(background.image, v.x, v.y)
	end
	love.graphics.setColor(255, 255, 255, 255)
	if game.state == "title" then
		Menu.draw()
	elseif game.state == "game" then
		Game.draw()
		--love.graphics.setCanvas()
		--love.graphics.draw(canvas, 0, 0, math.rad(0), 1, window.vscale)
	elseif game.state == "game over" then
		GameOver.draw()
	elseif game.state == "congratulations" then
		Congrats.draw()
	elseif game.state == "score" then
		if scores.type == "high" then
			Score.draw("FANTASTIC!", "You made an high-score!")
		elseif scores.type == "low" then
			Score.draw("COMMISERATIONS...", "You made a low-score...")
		end
	elseif game.state == "editor" then
		Editor.draw()
	elseif game.state == "test" then
		Test.draw()
	end
	--Debug.draw()
end
function love.keypressed(key)
	if key == "d" then
		--debug.debug()
	end
	if game.state == "game" then
		Game.keypressed(key)
	elseif game.state == "game over" then

	elseif game.state == "congratulations" then
		Congrats.keypressed(key)
	elseif game.state == "title" then
		Menu.keypressed(key)
	elseif game.state == "editor" then
		Editor.keypressed(key)
	elseif game.state == "score" then
		Score.keypressed(key)
	end
end
function love.mousepressed(x, y, btn)
	if game.state == "title" then
		Menu.mousepressed(x, y, btn)
	elseif game.state == "test" then
		Test.mousepressed(x, y, btn)
	elseif game.state == "game" then
		Game.mouseClicked(x, y, btn)
	elseif game.state == "game over" then
		GameOver.mousepressed(x, y, btn)
	elseif game.state == "congratulations" then
		if btn == "l" then
			Game.checkScore()
		end
	elseif game.state == "editor" then
		Editor.mousepressed(x, y, btn)
	end
end
function on_collision(dt, shape_a, shape_b, mtv_x, mtv_y)
	if game.state == "title" then
		if shape_a.type == "pointer" then
			pointer.over = shape_b.type
			menu.dialog.x, menu.dialog.y = shape_b:center()
			Menu.setOver(shape_b)
		elseif shape_b.type == "pointer" then
			pointer.over = shape_a.type
			menu.dialog.x, menu.dialog.y = shape_a:center()
			Menu.setOver(shape_a)
		end
	elseif game.state == "game" or game.state == "test" then
		if game.pause == true then
			if shape_a.type == "pointer" then
				game.pointer = shape_b.type
			elseif shape_b.type == "pointer" then
				game.pointer = shape_a.type
			end
		else
			if shape_a.type == "ball" then
				Game.bounce(shape_a, shape_b, mtv_x, mtv_y)
			elseif shape_b.type == "ball" then
				Game.bounce(shape_b, shape_a, -mtv_x, -mtv_y)
			elseif shape_a.type == "paddle" then
				Game.catch(shape_b)
			elseif shape_b.type == "paddle" then
				Game.catch(shape_a)
			elseif shape_a.type == "bullet" then
				if shape_b.type == "brick" then
					Player.hit(shape_a, shape_b)
				end
			elseif shape_b.type == "bullet" then
				if shape_a.type == "brick" then
					Player.hit(shape_b, shape_a)
				end
			end
		end
	elseif game.state == "editor" then
		if shape_a.type == "pointer" then
			editor.pointer = shape_b.type
			Editor.setOver(shape_b)
		elseif shape_b.type == "pointer" then
			editor.pointer = shape_a.type
			Editor.setOver(shape_a)
		end
	elseif game.state == "game over" then
		if shape_a.type == "pointer" then
			continue.pointer = shape_b.type
		elseif shape_b.type == "pointer" then
			continue.pointer = shape_a.type
		end
	end
end
function collision_stop(dr, shape_a, shape_b)
	Z = false
	if game.state == "title" then
		pointer.over = "nothing"
	elseif game.state == "game" then
		game.pointer = "nothing"
	elseif game.state == "editor" then
		editor.pointer = "nothing"
		editor.dialog.over = nil
	elseif game.state == "game over" then
		continue.pointer = "nothing"
	end
end
function wallpaper(image, width, height)
	local t = {}
	t.image = image
	local w = t.image:getWidth()
	local h = t.image:getHeight()
	local max = (math.floor(width / w) + 1) * (math.floor(height / h) + 1)
	t.grid = {}	
	local x = -w
	local y = 0
	for i = 1, max do
		z = {}
		z.pos = i
		x = x + w
		if x > width then
			x = 0
			y = y + h
		end
		z.x = x
		z.y = y
		table.insert(t.grid, z)
	end
	return t
end
function levelload(i)
	if game.custom == true then
		local p = love.filesystem.getSaveDirectory()
	end
	if love.filesystem.exists(game.path.."/level"..i..".lua") == true then
		TEsound.play("sfx/jingles_PIZZA12.ogg")
		bricks = table.load(game.path.."/level"..i..".lua")
		build()
		game.patternIndex = game.patternIndex + 1
		if game.patternIndex > 3 then
			game.patternIndex = 1
		end
		background = wallpaper(game.pattern[game.patternIndex], 768, 768)
		if level % 5 == 0 then
			game.colorIndex = game.colorIndex + 1
			if game.colorIndex > 6 then
				game.colorIndex = 1
			end
		end
		game.red = game.color[game.colorIndex].r
		game.green = game.color[game.colorIndex].g
		game.blue = game.color[game.colorIndex].b
		player.perfect = true
	else
		menu.complete = true
		game.state = "congratulations"
	end
end
function printbonus(text, x, y)
	love.graphics.setFont(font.bonus)
	love.graphics.setColor(128, 128, 128, 128)
	love.graphics.print(text, x + 1, y + 1)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.print(text, x, y)
end
function build()
	local index = 0
	for i, v in ipairs(bricks) do
		if v ~= 0 then
			index = index + 1
			v.box = collider:addRectangle(v.x, v.y, 64, 32)
			collider:setPassive(v.box)
			v.box.type = "brick"
			if v.brick == "grey" then
				if game.difficulty == "easy" then
					v.box.hp = 2
				elseif game.difficulty == "normal" then
					v.box.hp = 3
				elseif game.difficulty == "hard" then
					v.box.hp = 5
				end
			elseif v.brick == "gold" then
				v.box.hp = -1
				index = index - 1
			else
				v.box.hp = 1
			end
			collider:addToGroup("bricks", v.box)
		end
	end
	brick.count = index
	player.bonus = (math.ceil((30 + brick.count + level) / 10) * 10) + 0.9
end